package com.bpcl.lpg.core.cronjob;

import org.slf4j.LoggerFactory;
import de.hybris.platform.cronjob.model.CronJobModel;
import java.io.InputStream;
import java.util.Iterator;
import java.util.List;
import com.jcraft.jsch.ChannelSftp;
import java.io.IOException;
import com.jcraft.jsch.SftpException;
import com.jcraft.jsch.JSchException;
import org.apache.commons.io.FileUtils;
import de.hybris.platform.util.Config;
import com.bpcl.lpg.core.sftp.SFTPService;
import java.io.File;
import java.text.MessageFormat;
import com.bpcl.core.util.BPCLDateUtils;
import de.hybris.platform.cronjob.enums.CronJobStatus;
import de.hybris.platform.cronjob.enums.CronJobResult;
import de.hybris.platform.servicelayer.cronjob.PerformResult;
import com.bpcl.core.ym.notication.service.LpgV2NotificationEngineService;
import javax.annotation.Resource;
import com.bpcl.lpg.core.encryption.pgp.DefaultLpgPgpKeys;
import com.bpcl.core.ym.notication.service.NotificationEngineService;
import com.bpcl.lpg.core.utill.LpgCashMemoSTFPConnectionOperation;
import com.bpcl.lpg.core.service.LpgCashMemoImportService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.slf4j.Logger;
import com.bpcl.core.model.BpclCronjobModel;
import de.hybris.platform.servicelayer.cronjob.AbstractJobPerformable;

public class LpgCashMemoImportJob extends AbstractJobPerformable<BpclCronjobModel>
{
    private static final Logger LOG;
    private ConfigurationService configurationService;
    private LpgCashMemoImportService lpgCashMemoImportService;
    private LpgCashMemoSTFPConnectionOperation lpgCashMemoSTFPConnectionOperation;
    private NotificationEngineService notificationEngineService;
    @Resource(name = "defaultLpgPgpKeys")
    private DefaultLpgPgpKeys defaultLpgPgpKeys;
    private LpgV2NotificationEngineService lpgV2NotificationEngineService;
    
    public PerformResult perform(final BpclCronjobModel cronjob) {
        LpgCashMemoImportJob.LOG.info("LpgCashMemoImportJob perform method started");
        try {
            this.processInboundFileFromSFTP();
            return new PerformResult(CronJobResult.SUCCESS, CronJobStatus.FINISHED);
        }
        catch (final Exception ex) {
            LpgCashMemoImportJob.LOG.error("XML file Processing Exception : ", ex);
            final String body = MessageFormat.format("Cash Memo Import cronjob is failed due to exception at {0}", BPCLDateUtils.getCurrentDateAsString("dd-MMM-yy h:mm a"));
            this.lpgV2NotificationEngineService.sendEmail("FAILED Cash Memo Import Job Alert", body, this.configurationService.getConfiguration().getString("lpg.cash.memo.alert.email.receiver.id"));
            return new PerformResult(CronJobResult.FAILURE, CronJobStatus.ABORTED);
        }
    }
    
    private void processInboundFileFromSFTP() {
        try {
            final ChannelSftp sftpConnection = this.lpgCashMemoSTFPConnectionOperation.getSFTPConnection();
            LpgCashMemoImportJob.LOG.info("sftp Connection is success");
            final String sftpPath = this.configurationService.getConfiguration().getString("lpg.sftp.delivery.confirmation.inbound.path");
            final String namePrefix = this.configurationService.getConfiguration().getString("bpcl.cashmemo.report.filename");
            final List<String> listOfFileNames = this.lpgCashMemoSTFPConnectionOperation.dowbloadInboundFiles();
            for (String fileName : listOfFileNames) {
                if (fileName.startsWith(namePrefix)) {
                    sftpConnection.cd(sftpPath);
                    final String archivePath = this.configurationService.getConfiguration().getString("lpg.sftp.delivery.confirmation.inbound.archive.path");
                    final String encPath = this.configurationService.getConfiguration().getString("lpg.sftp.delivery.confirmation.inbound.enc.path");
                    sftpConnection.get(fileName, fileName);
                    LpgCashMemoImportJob.LOG.info("processing the file " + fileName);
                    final File csvFile = new File(fileName);
                    SFTPService.uploadStoredFile(sftpConnection, String.valueOf(csvFile), encPath);
                    final File originalCopy = csvFile;
                    if (Config.getBoolean("lpg.encryption.enable", false)) {
                        InputStream csvInputStream = null;
                        LpgCashMemoImportJob.LOG.info("Decrypting the file");
                        csvInputStream = this.defaultLpgPgpKeys.decryptAndVerify(csvFile.getAbsolutePath(), "lpg.encryption.privatekey", "lpg.encryption.pass");
                        FileUtils.copyInputStreamToFile(csvInputStream, csvFile);
                    }
                    this.lpgCashMemoImportService.processInboundFile(csvFile);
                    SFTPService.uploadStoredFile(sftpConnection, String.valueOf(originalCopy), archivePath);
                    sftpConnection.cd(sftpPath);
                    sftpConnection.rm(fileName);
                }
            }
            this.lpgCashMemoSTFPConnectionOperation.closeSFTPConnection();
            LpgCashMemoImportJob.LOG.info("sftp disConnection is success");
        }
        catch (final JSchException | SftpException | IOException e) {
            LpgCashMemoImportJob.LOG.error("Exception Occurred during inbound file reading", e);
            throw new RuntimeException(e);
        }
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public ConfigurationService getConfigurationService() {
        return this.configurationService;
    }
    
    public void setLpgCashMemoImportService(final LpgCashMemoImportService lpgCashMemoImportService) {
        this.lpgCashMemoImportService = lpgCashMemoImportService;
    }
    
    public LpgCashMemoImportService getLpgCashMemoImportService() {
        return this.lpgCashMemoImportService;
    }
    
    public void setLpgCashMemoSTFPConnectionOperation(final LpgCashMemoSTFPConnectionOperation lpgCashMemoSTFPConnectionOperation) {
        this.lpgCashMemoSTFPConnectionOperation = lpgCashMemoSTFPConnectionOperation;
    }
    
    public LpgCashMemoSTFPConnectionOperation getLpgCashMemoSTFPConnectionOperation() {
        return this.lpgCashMemoSTFPConnectionOperation;
    }
    
    public void setNotificationEngineService(final NotificationEngineService notificationEngineService) {
        this.notificationEngineService = notificationEngineService;
    }
    
    public LpgV2NotificationEngineService getLpgV2NotificationEngineService() {
        return this.lpgV2NotificationEngineService;
    }
    
    public void setLpgV2NotificationEngineService(final LpgV2NotificationEngineService lpgV2NotificationEngineService) {
        this.lpgV2NotificationEngineService = lpgV2NotificationEngineService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgCashMemoImportJob.class);
    }
}
